//**************************************************************************//
//																			//
// File:		Boot Loader.c												//
//																			//
// Project:		ADSP-2191 Host Port booting									//
//																			//
// Description: This file contains two functions that make up the core of	//
//				the software boot loader:									//
//				Function Transfer_Loader_File reads all values from the		//
//				loader file, analyses the information contained in the		//
//				headers and transfers the data blocks into the appropriate	//
//				memory spaces on the Target DSP.							//
//				The values are read sequentially from the loader file using	//
//				the function Read_Next_Loader_Value (platform dependent).	//
//				The values are transferred to the Target Host Port using	//
//				the low level access functions defined in file "Host Port	//
//				Access.c" (also platform dependent).						//
//				Function Start_Program_Execution clears some pre-defined	//
//				memory locations on the Target (handshake locations), and	//
//				starts program execution on the Target. The transferred		//
//				program will write defined values into the handshake		//
//				locations, which are then read back in function				//
//				Start_Program_Execution as confirmation for a successful	//
//				boot process.												//
//																			//
//**************************************************************************//

#include "Boot Loader.h"

//--------------------------------------------------------------------------//
// Function:	Transfer_Loader_File										//
//																			//
// Parameters:	None														//
//																			//
// Return:		Error code													//
//					EXIT_SUCCESS: Loader file transferred successfully		//
//					EXIT_FAILURE: Expected value not found in loader file	//
//																			//
// Description:	This function transfers the 16-bit loader file to the		//
//				Target DSP. It repeats the following steps until the final	//
//				block is processed:											//
//					- Read next header from file							//
//					- Configure Host Port DMA on Target						//
//					- Transfer data block or zero-init memory on Target		//
//--------------------------------------------------------------------------//
int Transfer_Loader_File(void)
{
	unsigned long ulTemp;
	unsigned int uiCounter;
	int iReturn_Value;							// return value from this function
	struct tBoot_Header Boot_Header;			// structure holding the current header

	// open loader file
	Open_Loader_File();

	// set default value for return code (overwritten in case of unexpected value in loader file)
	iReturn_Value = EXIT_SUCCESS;

	// read control word(s) from loader file
	Boot_Header.uiControl1 = Read_Next_Loader_Value();
	Boot_Header.uiControl2 = Read_Next_Loader_Value();

	// read all headers and data blocks; transfer data to Target DSP
	do
	{
		// read header for next data block
		Boot_Header.uiFlag			= Read_Next_Loader_Value();
		Boot_Header.uiAddress_LSW	= Read_Next_Loader_Value();
		Boot_Header.uiAddress_MSW	= Read_Next_Loader_Value();
		Boot_Header.uiCount			= Read_Next_Loader_Value();

		// configure Host Port DMA for Autobuffer mode on Target
		Write_Target_IO_Direct(H_HOSTD_CFG, 0x0010);	// enable Autobuffer Mode
		Write_Target_IO_Direct(H_HOSTD_SRP, Boot_Header.uiAddress_MSW);	// start page
		Write_Target_IO_Direct(H_HOSTD_SRA, Boot_Header.uiAddress_LSW);	// start address
		Write_Target_IO_Direct(H_HOSTD_CNT, Boot_Header.uiCount);	// number of words to transfer
		Write_Target_IO_Direct(H_HOSTD_CFG, 0x0017 | ((~Boot_Header.uiFlag & 0x0001) << 3));
												// enable Autobuffer Mode, interrupt and DMA;
												// transfer direction and data type
		
		// handle block according to Flag
		switch(Boot_Header.uiFlag & ALL_PM_DM)
		{
			case DATA_PM:						// transfer block of data to PM
			{
				for(uiCounter = 0; uiCounter < Boot_Header.uiCount; uiCounter++)
				{
					ulTemp = (unsigned long)Read_Next_Loader_Value() >> 8;
					ulTemp = ulTemp | ((unsigned long)Read_Next_Loader_Value() << 8);

					Write_Target_PM_DMA(ulTemp);
				}
				break;
			}
			case DATA_DM:						// transfer block of data to DM
			{
				for(uiCounter = 0; uiCounter < Boot_Header.uiCount; uiCounter++)
					Write_Target_DM_DMA(Read_Next_Loader_Value());
				break;
			}
			case ZERO_INIT_PM:					// initialise part of PM to zero
			{	
				for(uiCounter = 0; uiCounter < Boot_Header.uiCount; uiCounter++)
					Write_Target_PM_DMA(0x000000);
				break;
			}
			case ZERO_INIT_DM:					// initialise part of DM to zero
			{
				for(uiCounter = 0; uiCounter < Boot_Header.uiCount; uiCounter++)
					Write_Target_DM_DMA(0x0000);
				break;
			}
			default:
				iReturn_Value = EXIT_FAILURE;	// unexpected value in loader file
				
		}
	}											// do until final block is reached or error occured
	while((!(Boot_Header.uiFlag & 0x0002)) & (iReturn_Value == EXIT_SUCCESS));

	// disable Autobuffer Mode for Target Host Port
	Write_Target_IO_Direct(H_HOSTD_CFG, 0x0000);

	// close loader file
	Close_Loader_File();

	return(iReturn_Value);
}

//--------------------------------------------------------------------------//
// Function:	Start_Program_Execution										//
//																			//
// Parameters:	None														//
//																			//
// Return:		Error code													//
//					EXIT_SUCCESS: Target DSP booted successfully			//
//					EXIT_FAILURE: Handshake value not found in Target memory//
//																			//
// Description:	This function is called after the loader file has been		//
//				transferred to the Target DSP. It clears the handshake		//
//				locations in Target memory, then starts program execution	//
//				by writing a '1' into the Host Port Semaphore A register on	//
//				the Target.													//
//				The program running on the Target will write defined values	//
//				into the handshake locations. If these values can be read	//
//				out by the Host, the Target booted successfully.			//
//--------------------------------------------------------------------------//
int Start_Program_Execution(void)
{
	volatile unsigned int ui_Delay;
	int i_Handshake_Value_DM;
	long l_Handshake_Value_PM;

	// clear handshake locations in Target memory
	Write_Target_IO_Direct(H_HPPR, 0x0000);		// set bus width to 16 bit
	Write_Target_DM_Direct(HANDSHAKE_ADDRESS_DM, 0x0000);
	Write_Target_IO_Direct(H_HPPR, 0x0002);		// set bus width to 24 bit
	Write_Target_PM_Direct(HANDSHAKE_ADDRESS_PM, 0x000000);

	// start program execution on Target
	Write_Target_IO_Direct(H_HPSMPHA, 0x0001);	// writing a '1' to Host Port Semaphore A register
												// will start the program on the ADSP-2191

	// wait for Target DSP program execution
	for(ui_Delay = 0; ui_Delay < 0xffff; ui_Delay++);

	// read values from handshake locations in Target memory
	Write_Target_IO_Direct(H_HPPR, 0x0000);		// set bus width to 16 bit
	i_Handshake_Value_DM = Read_Target_DM_Direct(HANDSHAKE_ADDRESS_DM);
	Write_Target_IO_Direct(H_HPPR, 0x0002);		// set bus width to 24 bit
	l_Handshake_Value_PM = Read_Target_PM_Direct(HANDSHAKE_ADDRESS_PM);

	// compare values read with expected values
	if((i_Handshake_Value_DM == HANDSHAKE_VALUE_DM) & (l_Handshake_Value_PM == HANDSHAKE_VALUE_PM))
	{
		return(EXIT_SUCCESS);
	}
	else
	{
		return(EXIT_FAILURE);
	}
}
